#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <unistd.h>
#include "common.h"
#include "log.h"
#include "token.h"

Logger::Logger(char *filename)
{
  fp = 0;
  append_mode = 0;
  put_level = -1;

  if (filename) {
    setFilename(filename);
  } else {
    *default_filename = 0;
  }
}

Logger::~Logger()
{
}

int Logger::open(char *filename)
{
  char *fmode = "wt";
  if (fp) {
    close();
    fmode = "at";
  }
  if (append_mode) fmode = "at";

  if (!filename) {
    if (!*default_filename) {
      fp = stderr;
      return TRUE;
    }
    filename = default_filename;
  } else {
    setFilename(filename);
  }
  fp = fopen(filename, fmode);
  if (!fp) return FALSE;
  return TRUE;
}

/* -----------------------------------------------------------------*
   =NAME=
      put - $B%m%0$r=PNO$9$k(B
   =SYNOPSIS=
      void Logger::put(const char *fmt, ...)

      ... - printf $B$HF1$8(B
 * ----------------------------------------------------------------- */
void Logger::put(const char *fmt, ...)
{
  int ret;
  va_list ap;

  if (!fp) {
    ret = open();
    if (!ret) return;
  }
  va_start(ap, fmt);

  putLogTime();
  if (put_level >= 0) {
    fprintf(fp, "(%d) ", put_level);
#ifdef SAFE
    put_level = -1;
#endif
  }

  vfprintf(fp, fmt, ap);
  va_end(ap);
#ifndef DONT_FFLUSH
  fflush(fp);
#endif
}

/* ----------------------------------------------------------------- *
   =NAME=
      putLogTime - $B;~4V$r=PNO$9$k(B
   =SYNOPSIS=
      void Logger::putLogTime()
 * ----------------------------------------------------------------- */
void Logger::putLogTime()
{
  time_t t;
  char *s;
  char buf[40];
  t = time(0);
  s = ctime(&t);
  STRCPY(buf, s);
  chomp(buf);
  fprintf(fp, "%s: [%d] ", buf, getpid());
}

void Logger::setFilename(char *filename)
{
  STRCPY(default_filename, filename);
}

void Logger::close()
{
  if (fp && fp != stderr) {
    put("Close log file\n");
    fclose(fp);
    fp = 0;
  }
}
